Package: digest
Authors@R: c(person("Dirk", "Eddelbuettel", role = c("aut", "cre"), email = "edd@debian.org",
                    comment = c(ORCID = "0000-0001-6419-907X")),
             person("Antoine", "Lucas", role="ctb"),
             person("Jarek", "Tuszynski", role="ctb"),
             person("Henrik", "Bengtsson", role="ctb", comment = c(ORCID = "0000-0002-7579-5165")),
             person("Simon", "Urbanek", role="ctb", comment = c(ORCID = "0000-0003-2297-1732")),
             person("Mario", "Frasca", role="ctb"),
             person("Bryan", "Lewis", role="ctb"),
             person("Murray", "Stokely", role="ctb"),
             person("Hannes", "Muehleisen", role="ctb"),
             person("Duncan", "Murdoch", role="ctb"),
             person("Jim", "Hester", role="ctb"),
             person("Wush", "Wu", role="ctb", comment = c(ORCID = "0000-0001-5180-0567")),
             person("Qiang", "Kou", role="ctb", comment = c(ORCID = "0000-0001-6786-5453")),
             person("Thierry", "Onkelinx", role="ctb", comment = c(ORCID = "0000-0001-8804-4216")),
             person("Michel", "Lang", role="ctb", comment = c(ORCID = "0000-0001-9754-0393")),
             person("Viliam", "Simko", role="ctb"),
             person("Kurt", "Hornik", role="ctb", comment = c(ORCID = "0000-0003-4198-9911")),
             person("Radford", "Neal", role="ctb", comment = c(ORCID = "0000-0002-2473-3407")),
             person("Kendon", "Bell", role="ctb", comment = c(ORCID = "0000-0002-9093-8312")),
             person("Matthew", "de Queljoe", role="ctb"),
             person("Dmitry", "Selivanov", role="ctb"),
             person("Ion", "Suruceanu", role="ctb"),
             person("Bill", "Denney", role="ctb"),
             person("Dirk", "Schumacher", role="ctb"),
             person("András", "Svraka", role="ctb"),
             person("Sergey", "Fedorov", role="ctb"),
             person("Will", "Landau", role="ctb", comment = c(ORCID = "0000-0003-1878-3253")),
             person("Floris", "Vanderhaeghe", role="ctb", comment = c(ORCID = "0000-0002-6378-6229")),
             person("Kevin", "Tappe", role="ctb"),
             person("Harris", "McGehee", role="ctb"),
             person("Tim", "Mastny", role="ctb"),
             person("Aaron", "Peikert", role="ctb", comment = c(ORCID = "0000-0001-7813-818X")),
             person("Mark", "van der Loo", role="ctb", comment = c(ORCID = "0000-0002-9807-4686")),
             person("Chris", "Muir", role="ctb", comment = c(ORCID = "0000-0003-2555-3878")),
             person("Moritz", "Beller", role="ctb", comment = c(ORCID = "0000-0003-4852-0526")),
             person("Sebastian", "Campbell", role="ctb"),
             person("Winston", "Chang", role="ctb", comment = c(ORCID = "0000-0002-1576-2126")),
             person("Dean", "Attali", role="ctb", comment = c(ORCID = "0000-0002-5645-3493")),
             person("Michael", "Chirico", role="ctb", comment = c(ORCID = "0000-0003-0787-087X")),
             person("Kevin", "Ushey", role="ctb"))
Version: 0.6.37
Date: 2024-08-19
Title: Create Compact Hash Digests of R Objects
Description: Implementation of a function 'digest()' for the creation of hash
 digests of arbitrary R objects (using the 'md5', 'sha-1', 'sha-256', 'crc32',
 'xxhash', 'murmurhash', 'spookyhash', 'blake3', 'crc32c', 'xxh3_64', and 'xxh3_128'
 algorithms) permitting easy comparison of R language objects, as well as functions
 such as'hmac()' to create hash-based message authentication code. Please note that
 this package is not meant to be deployed for cryptographic purposes for which more
 comprehensive (and widely tested) libraries such as 'OpenSSL' should be used.
URL: https://github.com/eddelbuettel/digest,
        https://dirk.eddelbuettel.com/code/digest.html
BugReports: https://github.com/eddelbuettel/digest/issues
Depends: R (>= 3.3.0)
Imports: utils
License: GPL (>= 2)
Suggests: tinytest, simplermarkdown
VignetteBuilder: simplermarkdown
Encoding: UTF-8
NeedsCompilation: yes
Packaged: 2024-08-19 12:16:05 UTC; edd
Author: Dirk Eddelbuettel [aut, cre] (<https://orcid.org/0000-0001-6419-907X>),
  Antoine Lucas [ctb],
  Jarek Tuszynski [ctb],
  Henrik Bengtsson [ctb] (<https://orcid.org/0000-0002-7579-5165>),
  Simon Urbanek [ctb] (<https://orcid.org/0000-0003-2297-1732>),
  Mario Frasca [ctb],
  Bryan Lewis [ctb],
  Murray Stokely [ctb],
  Hannes Muehleisen [ctb],
  Duncan Murdoch [ctb],
  Jim Hester [ctb],
  Wush Wu [ctb] (<https://orcid.org/0000-0001-5180-0567>),
  Qiang Kou [ctb] (<https://orcid.org/0000-0001-6786-5453>),
  Thierry Onkelinx [ctb] (<https://orcid.org/0000-0001-8804-4216>),
  Michel Lang [ctb] (<https://orcid.org/0000-0001-9754-0393>),
  Viliam Simko [ctb],
  Kurt Hornik [ctb] (<https://orcid.org/0000-0003-4198-9911>),
  Radford Neal [ctb] (<https://orcid.org/0000-0002-2473-3407>),
  Kendon Bell [ctb] (<https://orcid.org/0000-0002-9093-8312>),
  Matthew de Queljoe [ctb],
  Dmitry Selivanov [ctb],
  Ion Suruceanu [ctb],
  Bill Denney [ctb],
  Dirk Schumacher [ctb],
  András Svraka [ctb],
  Sergey Fedorov [ctb],
  Will Landau [ctb] (<https://orcid.org/0000-0003-1878-3253>),
  Floris Vanderhaeghe [ctb] (<https://orcid.org/0000-0002-6378-6229>),
  Kevin Tappe [ctb],
  Harris McGehee [ctb],
  Tim Mastny [ctb],
  Aaron Peikert [ctb] (<https://orcid.org/0000-0001-7813-818X>),
  Mark van der Loo [ctb] (<https://orcid.org/0000-0002-9807-4686>),
  Chris Muir [ctb] (<https://orcid.org/0000-0003-2555-3878>),
  Moritz Beller [ctb] (<https://orcid.org/0000-0003-4852-0526>),
  Sebastian Campbell [ctb],
  Winston Chang [ctb] (<https://orcid.org/0000-0002-1576-2126>),
  Dean Attali [ctb] (<https://orcid.org/0000-0002-5645-3493>),
  Michael Chirico [ctb] (<https://orcid.org/0000-0003-0787-087X>),
  Kevin Ushey [ctb]
Maintainer: Dirk Eddelbuettel <edd@debian.org>
Repository: CRAN
Date/Publication: 2024-08-19 14:10:07 UTC
Built: R 4.3.3; aarch64-apple-darwin20; 2025-01-25 21:35:12 UTC; unix
Archs: digest.so.dSYM
